//------------------------------------------------------------------------------
// Copyright (c) 2004, Perforce Software, Inc.  All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1.  Redistributions of source code must retain the above copyright
//     notice, this list of conditions and the following disclaimer.
// 
// 2.  Redistributions in binary form must reproduce the above copyright
//     notice, this list of conditions and the following disclaimer in the
//     documentation and/or other materials provided with the distribution.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL PERFORCE SOFTWARE, INC. BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//------------------------------------------------------------------------------
#define NEED_STAT
#include <clientapi.h>
#include <strtable.h>
#include <vararray.h>
#include "filelist.h"
#include "syncuser.h"

static StrRef	msgAddedAs( " - added as " );
static StrRef	msgReplacing( " - replacing " );
static StrRef	msgUpdating( " - updating " );
static StrRef	msgRefreshing( " - refreshing " );

//
// Only called when we run the sync. For each output message, find the local
// path in the message (if possible), and append it to the worklist. If the
// message is an error message, pop the last item off the worklist.
//
void
SyncUser::Message( Error *err )
{
    // Ensure default behaviour happens
    ClientUser::Message( err );

    if( !err->IsInfo() ) return;
    
    StrBuf	m;
    err->Fmt( &m, EF_PLAIN );
    StrBuf	file;
    const char *pos = 0;

    if( pos = m.Contains( msgAddedAs ) )
    {
	file.Set( m.Text(), pos - m.Text() );
    }
    else if( pos = m.Contains( msgReplacing ) )
    {
	file.Set( m.Text(), pos - m.Text() );
    }
    else if( pos = m.Contains( msgUpdating ) )
    {
	file.Set( m.Text(), pos - m.Text() );
    }
    else if( pos = m.Contains( msgRefreshing ) )
    {
	file.Set( m.Text(), pos - m.Text() );
    }
    else
    {
	skippedLast = 1;
	return;
    }

    skippedLast = 0;
    PushFile( &file );
}

//
// Handle errors during syncs
//
void
SyncUser::HandleError( Error *e )
{
    ClientUser::HandleError( e );
    if( !e->IsInfo() )
    {
	if( !skippedLast ) PopFile();
	return;
    }
}

//
// Only chmod files if we've got something in the worklist
//
int
SyncUser::ChmodFiles()
{
    return WorkList()->Count();
}
